# -*- coding: utf-8 -*-

"""
"""

__author__ = "Stefan Hendricks"

import requests
from datetime import datetime
from pathlib import Path
from loguru import logger
import numpy as np

from icedrift import GeoReferenceStation


class PolarsternAWIDashboardPos(object):

    def __init__(self, begin_date=None, end_date=None, output="csv"):
        """

        :param begin_date:
        :param end_date:
        :param output:
        """

        # Save input arguments
        self.begin_date = begin_date
        self.end_date = end_date
        self.output = output

        # --- Properties ---

        # The format of the REST output.
        # NOTE: The (default) value is specified in the `output` keyword to this method
        self.format_dict = {"csv": "text/csv",
                            "json": "application/json",
                            "tab": "text/tab-separated-values"}

        # Output format for the time stamp
        # NOTE: Most sensors do not report faster than 1 second, therefore there is no need
        #       for microseconds
        self.datetime_format = "%Y-%m-%dT%H:%M:%S"

        # The main URL to the dashboard REST interface
        self.base_url = "https://dashboard.awi.de/data-xxl/rest/data"

        # A list of sensors
        self.sensors = ["vessel:polarstern:hydrins_1:latitude$NRT",
                        "sensors=vessel:polarstern:hydrins_1:longitude$NRT",
                        "sensors=vessel:polarstern:hydrins_1:heading$NRT"]

        # Temporal resolution:
        self.aggregate_value = "second"

    def save(self, filename=None):
        """

        :param filename:
        :return:
        """

        # Construct the URL
        url = self._construct_url()
        logger.debug("url=%s" % url)

        # Download the websites response
        # TODO: Error management
        r = requests.get(url)

        # Post-process the content
        ascii_content = "\n".join(r.text.splitlines())
        if self.output == "csv":
            ascii_content = ascii_content.replace(";", ",")

        # Write output file
        open(filename, 'w').write(ascii_content)

    def _construct_url(self):
        """
        Create the url from the input
        :return:
        """

        # Add base url
        url = self.base_url+"?"

        # add sensors
        url += self.sensor_def

        # add output format
        url += "&format=%s" % self.format_dict[self.output]

        # add temporal resolution
        url += "&aggregate={}".format(self.aggregate_value)

        # add begin date (optional)
        if self.begin_date is not None:
            url += "&beginDate=%s" % self.begin_date.strftime(self.datetime_format)

        # add end date (optional)
        if self.end_date is not None:
            url += "&endDate=%s" % self.end_date.strftime(self.datetime_format)

        return url

    @property
    def sensor_def(self):
        """
        Constructs the sensor string from the list of sensors
        :return: (str) the sensor string in the form of sensors={...}&{...}&{...}
        """
        sensor_def = "sensors="+"&".join(self.sensors)
        return sensor_def


class PolarsternPOSDashboardData(object):
    """
    Interface to Polarstern data repository downloaded from the dashboard
    REST interface.
    """

    def __init__(self, repo_folder, filename_template="ps-pos-{year}-{month:02g}-{day:02g}.csv"):
        """

        :param repo_folder:
        :return:
        """

        # --- Properties ---

        # Lookup directory for the data files
        self.repo_folder = repo_folder

        # Filename template
        self.filename_template = filename_template

    def get_date(self, target_date):
        """
        Get a position object for a single (UTC) day
        :param target_date: datetime.date
        :return:
        """

        # Construct the file path of the target (daily) file
        filepath = self._get_filepath(target_date)

        # Read the file
        data = self._get_file_content(filepath)
        return data

    def _get_filepath(self, date):
        """
        Return the filename for a date
        :param date: datetime.date
        :return: pathlib.Path
        """
        filename = self.filename_template.format(year=date.year, month=date.month, day=date.day)
        filepath = Path(self.repo_folder) / filename
        return filepath

    @staticmethod
    def _get_file_content(filepath):
        """
        Read a file and return the data as position object
        :param filepath:
        :return:
        """

        # Converter of datetime strings to datetime object
        str2date = lambda x: datetime.strptime(x.decode("utf-8"), '%Y-%m-%dT%H:%M:%S.000')

        # Read csv content
        content = np.genfromtxt(filepath, delimiter=',', skip_header=1,
                                names="time, latitude, longitude, heading",
                                dtype=(object, float, float, float),
                                converters={0: str2date})

        # Create a georeference station
        data = GeoReferenceStation(content["time"], content["longitude"], content["latitude"], content["heading"])

        return data


